<?php

namespace ProjectManagers\ProjectManager\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use ProjectManagers\ProjectManager\Forms\Project as ProjectForm;
use ProjectManagers\ProjectManager\Http\Requests\ProjectRequest;
use ProjectManagers\ProjectManager\Interfaces\ProjectRepositoryInterface;
use ProjectManagers\ProjectManager\Repositories\Eloquent\Filters\ProjectResourceFilter;
use ProjectManagers\ProjectManager\Repositories\Eloquent\Presenters\ProjectListPresenter;

/**
 * Resource controller class for project.
 */
class ProjectResourceController extends BaseController
{

    /**
     * Initialize project resource controller.
     *
     *
     * @return null
     */
    public function __construct(ProjectRepositoryInterface $project)
    {
        parent::__construct();
        $this->form = ProjectForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('project_managers.project_manager.modules'), 'project_manager', guard_url('project_manager'));
        $this->repository = $project;
    }

    /**
     * Display a list of project.
     *
     * @return Response
     */
    public function index(ProjectRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(ProjectResourceFilter::class)
            ->setPresenter(ProjectListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('project_manager::project.names'))
            ->view('project_manager::project.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display project.
     *
     * @param Request $request
     * @param Model   $project
     *
     * @return Response
     */
    public function show(ProjectRequest $request, ProjectRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('project_manager::project.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('project_manager::project.show')
            ->output();
    }

    /**
     * Show the form for creating a new project.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(ProjectRequest $request, ProjectRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('project_manager::project.name'))
            ->view('project_manager::project.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new project.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ProjectRequest $request, ProjectRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('project_manager::project.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('project_manager/project/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/project_manager/project'))
                ->redirect();
        }

    }

    /**
     * Show project for editing.
     *
     * @param Request $request
     * @param Model   $project
     *
     * @return Response
     */
    public function edit(ProjectRequest $request, ProjectRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('project_manager::project.name'))
            ->view('project_manager::project.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the project.
     *
     * @param Request $request
     * @param Model   $project
     *
     * @return Response
     */
    public function update(ProjectRequest $request, ProjectRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('project_manager::project.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('project_manager/project/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('project_manager/project/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the project.
     *
     * @param Model   $project
     *
     * @return Response
     */
    public function destroy(ProjectRequest $request, ProjectRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('project_manager::project.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('project_manager/project/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('project_manager/project/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
